@@ -17,6 +17,7 @@
 use std::error::Error;
 use std::fmt;
 use std::io;
+use std::str::FromStr;
 
 use hyper::Error as HttpClientError;
 use hyper::client::Response;
@@ -28,11 +29,6 @@ use serde_json::Error as JsonError;
 /// Error from an OpenStack API call.
 #[derive(Debug)]
 pub enum ApiError {
-    /// Insufficient credentials passed to make authentication request.
-    ///
-    /// Contains the error message.
-    InsufficientCredentials(String),
-
     /// Requested service endpoint was not found.
     ///
     /// Contains the failed endpoint name.
@@ -41,7 +37,7 @@ pub enum ApiError {
     /// Invalid value passed to one of paremeters.
     ///
     /// Contains the error message.
-    InvalidParameterValue(String),
+    InvalidInput(String),
 
     /// Generic HTTP error.
     HttpError(StatusCode, Response),
@@ -52,9 +48,6 @@ pub enum ApiError {
     /// JSON parsing failed.
     InvalidJson(JsonError),
 
-    /// Malformed response.
-    MalformedResponse(String),
-
     /// Malformed API version.
     #[allow(missing_docs)]
     InvalidApiVersion { value: String, message: String },
@@ -88,22 +81,27 @@ pub enum ApiVersionRequest {
     Choice(Vec<ApiVersion>)
 }
 
+/// Sorting request.
+#[derive(Debug, Clone)]
+pub enum Sort<T: Into<String>> {
+    /// Sorting by given field in ascendant order.
+    Asc(T),
+    /// Sorting by given field in descendant order.
+    Desc(T)
+}
+
 
 impl fmt::Display for ApiError {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match *self {
-            ApiError::InsufficientCredentials(ref msg) =>
-                write!(f, "Insufficient credentials provided: {}", msg),
             ApiError::EndpointNotFound(ref endp) =>
                 write!(f, "Requested endpoint {} was not found", endp),
-            ApiError::InvalidParameterValue(ref msg) =>
-                write!(f, "Passed parameters are invalid: {}", msg),
+            ApiError::InvalidInput(ref msg) =>
+                write!(f, "Input value(s) are invalid: {}", msg),
             ApiError::HttpError(status, ..) =>
                 write!(f, "HTTP error {}", status),
             ApiError::ProtocolError(ref e) => fmt::Display::fmt(e, f),
             ApiError::InvalidJson(ref e) => fmt::Display::fmt(e, f),
-            ApiError::MalformedResponse(ref msg) =>
-                write!(f, "Malformed response received: {}", msg),
             ApiError::InvalidApiVersion { value: ref val, message: ref msg } =>
                 write!(f, "{} is not a valid API version: {}", val, msg),
             ApiError::UnsupportedApiVersion {
@@ -117,17 +115,12 @@ impl fmt::Display for ApiError {
 impl Error for ApiError {
     fn description(&self) -> &str {
         match *self {
-            ApiError::InsufficientCredentials(..) =>
-                "Insufficient credentials provided",
             ApiError::EndpointNotFound(..) =>
                 "Requested endpoint was not found",
-            ApiError::InvalidParameterValue(..) =>
-                "Invalid values passed for parameters",
+            ApiError::InvalidInput(..) => "Invalid value(s) provided",
             ApiError::HttpError(..) => "HTTP error",
             ApiError::ProtocolError(ref e) => e.description(),
             ApiError::InvalidJson(ref e) => e.description(),
-            ApiError::MalformedResponse(..) =>
-                "Malformed response received",
             ApiError::InvalidApiVersion { .. } =>
                 "Invalid API version",
             ApiError::UnsupportedApiVersion { .. } =>
@@ -174,40 +167,49 @@ impl fmt::Display for ApiVersion {
     }
 }
 
-fn parse_component(component: &str, value: &String, message: &str)
+fn parse_component(component: &str, value: &str, message: &str)
         -> ApiResult<u16> {
     match component.parse() {
         Ok(val) => Ok(val),
         Err(..) => Err(ApiError::InvalidApiVersion {
-            value: value.clone(),
+            value: String::from(value),
             message: String::from(message)
         })
     }
 }
 
-impl ApiVersion {
-    /// Parse string, yielding an API version.
-    pub fn parse<S: Into<String>>(value: S) -> ApiResult<ApiVersion> {
-        let s = value.into();
+impl FromStr for ApiVersion {
+    type Err = ApiError;
+
+    fn from_str(s: &str) -> ApiResult<ApiVersion> {
         let parts: Vec<&str> = s.split('.').collect();
 
         if parts.len() != 2 {
             return Err(ApiError::InvalidApiVersion {
-                value: s.clone(),
+                value: String::from(s),
                 message: String::from("Expected format X.Y")
             });
         }
 
-        let major = try!(parse_component(parts[0], &s,
+        let major = try!(parse_component(parts[0], s,
                                          "First component is not a number"));
 
-        let minor = try!(parse_component(parts[1], &s,
+        let minor = try!(parse_component(parts[1], s,
                                          "Second component is not a number"));
 
         Ok(ApiVersion(major, minor))
     }
 }
 
+impl<T: Into<String>> Into<(String, String)> for Sort<T> {
+    fn into(self) -> (String, String) {
+        match self {
+            Sort::Asc(val) => (val.into(), String::from("asc")),
+            Sort::Desc(val) => (val.into(), String::from("desc"))
+        }
+    }
+}
+
 #[cfg(test)]
 pub mod test {
     use super::ApiVersion;
